package anywheresoftware.b4a.objects;

import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.DependsOn;
import anywheresoftware.b4a.BA.Events;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;
import dji.common.error.DJIError;
import dji.common.error.DJISDKError;
import dji.common.realname.AppActivationState;
import dji.common.realname.AppActivationState.AppActivationStateListener;
import dji.sdk.base.BaseProduct;
import dji.sdk.sdkmanager.DJISDKManager;
import dji.sdk.useraccount.UserAccountManager;

@ShortName("DJISDKManager")
@Version(4.5f)
@DependsOn(values={"dji-sdk-4.5.aar","dji-uxsdk-4.5.aar", "com.android.support:support-v4"})
@Events(values={"RegisteredResult (Success As Boolean, ErrorMessage As String)", "ActivationStateChanged (State As String)",
		"ProductChanged (AircraftData As Object)", "ProductConnectivityChanged (Connected As Boolean)", "ComponentConnectivityChanged (Connected As Boolean)"
		, "ProductName (Success As Boolean, Name As String)"})
		public class DJISDKManagerWrapper  {
	@Hide
	public DJISDKManager manager;
	private BA ba;
	private String eventName;
	private boolean activationListenerAdded;
	public boolean IsInitialized() {
		return manager != null;
	}
	/**
	 * Initializes the DJI SDK. The RegisteredResult event will be raised.
	 *Note that an internet connection is required on the first run.
	 */
	public void Initialize(final BA ba, String EventName) {
		manager = DJISDKManager.getInstance();
		this.ba = ba;
		this.eventName = EventName.toLowerCase(BA.cul);
		DJISDKManager.getInstance().registerApp(ba.context, new DJISDKManager.SDKManagerCallback() {


			@Override
			public void onProductChange(BaseProduct paramBaseProduct1,
					BaseProduct paramBaseProduct2) {
				ba.raiseEventFromDifferentThread(DJISDKManagerWrapper.this, null, 0, eventName + "_productchanged", true, new Object[] {paramBaseProduct2});

			}

			@Override
			public void onRegister(DJIError paramDJIError) {
				ba.raiseEventFromDifferentThread(DJISDKManagerWrapper.this, null, 0, eventName + "_registeredresult", true, 
						new Object[] {paramDJIError == DJISDKError.REGISTRATION_SUCCESS,
						paramDJIError.getDescription()});
				if (activationListenerAdded == false && manager.getAppActivationManager() != null) {
					activationListenerAdded = true;
					manager.getAppActivationManager().addAppActivationStateListener(new AppActivationStateListener() {

						@Override
						public void onUpdate(AppActivationState arg0) {
							ba.raiseEventFromDifferentThread(DJISDKManagerWrapper.this, null, 0, eventName + "_activationstatechanged", true, new Object[] {arg0.toString()});
						}

					});
				}
			}

		});

	}

	/**
	 * Tries to connect to the product. The ProductChanged event will be raised.
	 */
	public void StartConnectionToProduct() {
		manager.startConnectionToProduct();
	}
	public boolean getRegistered() {
		return manager.hasSDKRegistered();
	}
	/**
	 * Returns the activation state.
	 * One of the following values: NOT_SUPPORTED, LOGIN_REQUIRED, ACTIVATED, UNKNOWN
	 * Returns an empty string if the SDK was not registered.
	 */
	public String getActivationState() {
		if (manager.getAppActivationManager() == null)
			return "";
		return String.valueOf(manager.getAppActivationManager().getAppActivationState());
	}
	/**
	 * Returns the aircraft binding state.
	 * One of the following values: NOT_SUPPORTED, INITIAL, BOUND, NOT_REQUIRED, UNBOUND , UNBOUND_BUT_CANNOT_SYNC,  UNKNOWN
	 * Returns an empty string if the SDK was not registered.
	 */
	public String getAircraftBindingState() {
		if (manager.getAppActivationManager() == null)
			return "";
		return String.valueOf(manager.getAppActivationManager().getAircraftBindingState());
	}
	@ShortName("DJIUserManager")
	public static class DJIUserManager extends DJIBaseComponentWrapper<UserAccountManager> {
		public void Initialize(final BA ba, String EventName) {
			setObject(UserAccountManager.getInstance());
			super.initialize(ba, EventName);
		}
		public Object Login() {
			B4ACompletionCallback cc = new B4ACompletionCallback();
			getObject().logIntoDJIUserAccount(getBA().context, cc);
			return cc;
		}
		public Object Logout() {
			B4ACompletionCallback cc = new B4ACompletionCallback();
			getObject().logoutOfDJIUserAccount(cc);
			return cc;
		}
		public String getState() {
			return String.valueOf(getObject().getUserAccountState());
		}

	}

}
