package anywheresoftware.b4a.objects;

import dji.common.error.DJIError;
import dji.common.mission.hotpoint.HotpointHeading;
import dji.common.mission.hotpoint.HotpointMission;
import dji.common.mission.hotpoint.HotpointMissionEvent;
import dji.common.mission.hotpoint.HotpointStartPoint;
import dji.common.model.LocationCoordinate2D;
import dji.sdk.mission.MissionControl;
import dji.sdk.mission.hotpoint.HotpointMissionOperator;
import dji.sdk.mission.hotpoint.HotpointMissionOperatorListener;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Events;
import anywheresoftware.b4a.BA.ShortName;

@ShortName("DJIHotpointMissionOperator")
@Events(values={"MissionStart", 
		"MissionFinish (Error As String)",
		"MissionUpdate (Error As String, CurrentState As String, PreviousState As String, Radius As Float)"}) 
public class HotpointMissionOperatorWrapper extends DJIBaseComponentWrapper<HotpointMissionOperator>{
	
	/**
	 * Initializes the mission operator.
	 */
	public void Initialize(final BA ba, String EventName) {
		setObject(MissionControl.getInstance().getHotpointMissionOperator());
		super.initialize(ba, EventName);
		getObject().addListener(new HotpointMissionOperatorListener() {

			@Override
			public void onExecutionFinish(DJIError paramDJIError) {
				ba.raiseEventFromUI(getObject(), getEventName() + "_missionfinish", paramDJIError == null ? "" : String.valueOf(paramDJIError));
			}

			@Override
			public void onExecutionStart() {
				ba.raiseEventFromUI(getObject(), getEventName() + "_missionstart");
			}

			@Override
			public void onExecutionUpdate(HotpointMissionEvent arg0) {
				ba.raiseEventFromUI(getObject(), getEventName() + "_missionupdate", arg0.getError() == null ? "" : String.valueOf(arg0.getError()),arg0.getCurrentState().getName().toString(),arg0.getPreviousState().getName().toString(),arg0.getRadius());
				
			}
			
		});
	}
	/**
	 * Starts a mission. The Result event will be raised.
	 */
	public Object StartMission(double hotpointLat, double hotpointLon, double altitude, double radius, float angularVelocity, boolean isClockwise,
      String startPoint, String  heading) {
		
		LocationCoordinate2D hotpoint = new LocationCoordinate2D(hotpointLat, hotpointLon);
		B4ACompletionCallback cc = new B4ACompletionCallback();
		getObject().startMission(new HotpointMission(hotpoint, altitude, radius, angularVelocity, isClockwise,
				HotpointStartPoint.valueOf(startPoint),HotpointHeading.valueOf(heading)), cc);
		return cc;
	}
	/**
	 * Resumes a mission. The Result event will be raised.
	 */
	public Object Resume() {
		B4ACompletionCallback cc = new B4ACompletionCallback();
		getObject().resume(cc);
		return cc;
	}
	/**
	 * Pauses a mission. The Result event will be raised.
	 */
	public Object Pause() {
		B4ACompletionCallback cc = new B4ACompletionCallback();
		getObject().pause(cc);
		return cc;
	}
	/**
	 * Stops a mission. The Result event will be raised.
	 */
	public Object Stop() {
		B4ACompletionCallback cc = new B4ACompletionCallback();
		getObject().stop(cc);
		return cc;
	}
	
	
}
